
/***************************************************************************
 *   This file is part of Aspect, a simple PEC tool.                       *
 *                                                                         *
 *   Copyright (C) 2006-2007 by Wolfgang Hoffmann <woho@woho.de>           *
 *                                                                         *
 *   This program is free software, licensed under the GPL v2.             *
 *   See the file COPYING for more details.                                *
 ***************************************************************************/


#ifndef ASPECT_MAINWINDOW_H
#define ASPECT_MAINWINDOW_H


#include <QMainWindow>
#include <QList>

class QGraphicsScene;
class QStandardItem;
class QStandardItemModel;


namespace Ui { class MainWindow; };


class PeValue
    {
public:
    double m_dfTime;
    double m_dfStarPos;
    double m_dfWormPos;
    double m_dfPe;
    };


class Period
    {
public:
    int m_nRowBeg;
    int m_nRowEnd;
    double m_dfPeMin;
    double m_dfPeMax;
    };


class MainWindow: public QMainWindow
    {
    Q_OBJECT

public:
    MainWindow();
    ~MainWindow();

public slots:
    void on_actionQuit_triggered();
    void on_actionAbout_Aspect_triggered();
    void on_actionAbout_Qt_triggered();
    void on_actionOpen_PE_Data_triggered();
    void on_actionOpen_PEC_Table_triggered();
    void on_actionSave_PEC_Table_triggered();
    void on_actionCalculate_PEC_triggered();
    void on_aquiPixelScale_valueChanged(double dfScale);
    void on_aquiStarDec_valueChanged(double dfDeclination);
    void on_aquiDrift_valueChanged(double dfDrift);
    void on_wormTime_valueChanged(double dfTime);
    void on_wormPos_valueChanged(int nPosition);
    void on_wormSpeed_activated(int nSpeed);
    void on_wormGearTeeth_valueChanged(int nTeeth);
    void on_wormHemisphere_activated(int nHemisphere);
    void on_wormRotation_activated(int nRotation);

private:
    void openPeFile(const QString &qsFileName);
    void openPecFile(const QString &qsFileName);
    void savePecFile(const QString &qsFileName);

private slots:
    void changePeItem(QStandardItem *pItem);
    void changePecItem(QStandardItem *pItem);

private:
    void recalcPeData();
    void recalcPeriods();
    void recalcPeChart();
    void recalcPecChart();

private:
    Ui::MainWindow *m_pUi;
    QList<PeValue> *m_pPeData;
    QList<Period> *m_pPeriod;
    int m_nPecFactor;
    QList<int> *m_pPecTable;
    QStandardItemModel *m_pPeModel;
    QStandardItemModel *m_pPecModel;
    QGraphicsScene *m_pPeChart;
    QGraphicsScene *m_pPecChart;
    };


#endif // ASPECT_MAINWINDOW_H

